<?php
#!/usr/local/bin/php

/* $Id$ */

/**
 * Conversion script from phpbb 2.0.18 to Drupal 4.6.5.
 * 
 * Written by: John Hwang
 * Date: 2006-01-12
 *
 * The following modules are required to be not only installed but also enabled
 *
 * forum
 * node
 * comment
 * user
 * profile
 * taxonomy
 * upload (optional, need for attachments)
 * comment_upload (optional, need for attachments)
 * 
 * Notes:
 * 1. You have to specify where your PHPBB database and Drupal databases are.
 *    a. Open setting.php
 *    b. Change the default dsn_url to
 *       $db_url['default'] = 'mysql://username:password@localhost/drupal_database';
 *       $db_url['phpbb'] = 'mysql://username:password@localhost/phpbb_database';
 *    Because this script uses Drupal's db_set_active() to switch between databases
 *    The advantage of this method is that now you are not required to have the tables
 *    in the same database.  It doesn't even require that either databases be on the 
 *    same machine.
 * 2. Adjust the $TIME_LIMIT for an appropriately large value.  The larger the 
 *    data you're trying to import, the higher the value should be.
 * 3. To import attachments, change $PHPBB2DRUPAL_IMPORT_ATTACHMENTS's value to TRUE.
 * 4. Once you enable the forum module, it's a good idea to go to adminster->forums 
 *    so that the forum module creates a vocabulary for your forum.  If not, the script
 *    will automatically create a vocabulary name "Forums."
 * 5. The script does not support PHPBB2's attachment thumbnails
 * 6. To import attachments, install and Enable both the new comment.module and the comment_upload.module
 * 7. You must manually move/link the contents of the old attachment directory to drupal's files 
 *    directory of your drupal installation.  It import script does not check if the
 *    file actually exists.  It won't move it for you either.  It only imports whatever
 *    is in your PHPBB2 database.
 */

// error_reporting(E_ALL);
// ini_set('display_errors', TRUE);

$PHPBB2DRUPAL_IMPORT_ATTACHMENTS = FALSE;
$PHPBB2DRUPAL_TIME_LIMIT = 300;                  // 5 minutes
$PHPBB2DRUPAL_FORUM_NAME = 'Forums';

// Disable access checking?
$access_check = TRUE;

// Adjust how long you want the script to run...
if (!ini_get("safe_mode")) {
  set_time_limit($PHPBB2DRUPAL_TIME_LIMIT);  // 5 Minutes still might not be enough
}

if (isset($_GET["op"])) {
  include_once "includes/bootstrap.inc";
  include_once "includes/common.inc";

  // Access check:
  if (($access_check == 0) || ($user->uid == 1)) {
    phpbb2drupal_page();
  }
  else {
    print phpbb2drupal_page_header("Access denied");
    print "<p>Access denied.  You are not authorized to access this page.  Please log in as the admin user (the first user you created). If you cannot log in, you will have to edit <code>update.php</code> to bypass this access check.  To do this:</p>";
    print "<ol>";
    print " <li>With a text editor find the update.php file on your system. It should be in the main Drupal directory that you installed all the files into.</li>";
    print " <li>There is a line near top of update.php that says <code>\$access_check = TRUE;</code>. Change it to <code>\$access_check = FALSE;</code>.</li>";
    print " <li>As soon as the script is done, you must change the update.php script back to its original form to <code>\$access_check = TRUE;</code>.</li>";
    print " <li>To avoid having this problem in future, remember to log in to your website as the admin user (the user you first created) before you backup your database at the beginning of the update process.</li>";
    print "</ol>";

    print phpbb2drupal_page_footer();
  }
}
else {
  phpbb2drupal_info();
}

function phpbb2drupal_info() {
  print phpbb2drupal_page_header("Drupal PHPBB2 Import");
?>
  <h3>Before doing anything, <strong>backup your database!</strong> This process will change your database and its values, and some things might get lost!!</h3>
  
  <h3>Notes:</h3>

  The following modules are required to be not only installed but also enabled:
  <ul>
    <li>forum</li>
    <li>node</li>
    <li>comment</li>
    <li>user</li>
    <li>profile</li>
    <li>taxonomy</li>
    <li>upload (optional)</li>
    <li>comment_upload (optional)</li>
  </ul>

  <ol>
  <li>You have to specify where your PHPBB database and Drupal databases are.</li>
      <ol>
      <li>Open setting.php</li>
      <li>Change the default dsn_url to</li>
          <ul>
          <li>$db_url['default'] = 'mysql://username:password@localhost/drupal_database';</li>
          <li>$db_url['phpbb'] = 'mysql://username:password@localhost/phpbb_database';</li>
          </ul>
      <li>Because this script uses Drupal's db_set_active() to switch between databases.  The advantage of this method is that now you are not required to have the tables in the same database.  It doesn't even require that either databases be on the same machine.</li>
      </ol>
  <li>Adjust the $PHPBB2DRUPAL_TIME_LIMIT for an appropriately large value.  The larger the data you're trying to import, the higher the value should be.</li>
  <li>Once you enable the forum module, it's a good idea to go to adminster->forums so that the forum module creates a vocabulary for your forum.  If not, the script will automatically create a vocabulary name "Forums."</li>
  <li>To import attachments, change $PHPBB2DRUPAL_IMPORT_ATTACHMENTS's value to TRUE.
  <li>The script does not support PHPBB2's attachment thumbnails</li>
  <li>Install and Enable both the new comment.module and the comment_upload.module</li>
  <li>You must manually move/link the contents of the old attachment directory to drupal's files directory of your drupal installation.  It import script does not check if the file actually exists.  It won't move it for you either.  It only imports whatever is in your PHPBB2 database.</li>
  </ol>

  <h3><a href="phpbb2drupal_import.php?op=import">Begin the import process</a></h3>
<?php
  print phpbb2drupal_page_footer();
}

function phpbb2drupal_page() {
  if (isset($_POST['edit'])) {
    $edit = $_POST['edit'];
  }

  $PHPBB2DRUPAL_FUNCTIONS = array(
    'users' => 'Import Users',
    'categories' => 'Import Categories',
    'topics' => 'Import Topics',
    'posts' => 'Import Posts',
    'cleanup' => 'Clean Up'
  );

  print phpbb2drupal_page_header("Drupal PHPBB2 import");
  $links[] = "<a href=\"index.php\">main page</a>";
  $links[] = "<a href=\"index.php?q=admin\">administration pages</a>";
  print theme("item_list", $links);
  
  $action = $edit['import'];
  switch ($action) {
    case "users":
      phpbb2drupal_import_users();
      $selected = 'categories';
      break;

    case "categories":
      phpbb2drupal_import_categories();
      $selected = 'topics';
      break;

    case "topics":      
      phpbb2drupal_import_topics();
      $selected = 'posts';
      break;
    
    case "posts":
      phpbb2drupal_import_posts();
      $selected = 'cleanup';
      break;

    case "cleanup":
      phpbb2drupal_import_cleanup();
      $selected = 'done';
      break;
    
    default:
       // make update form and output it.
      $selected = 'users';
      break;
  }
  
  if($selected == 'done') {
    print "<h2>Congratulations.  Import Finished</h2>";
  } else {
    $form = form_select("Next import to perform", "import", $selected, $PHPBB2DRUPAL_FUNCTIONS);
    $form .= form_submit("Import");
    print form($form);
  }
  print phpbb2drupal_page_footer();
}

/**
 * User Import Functions
 */
function phpbb2drupal_import_users() {

  // check if the user database has been successfully imported
  db_set_active('default');
  if(variable_get('phpbb2drupal_import_user_successful', 0)) return;

  if(!variable_get('phpbb2drupal_import_user_started', 0)) {
    // create temporary tables
    db_set_active('default');
    db_query("DROP TABLE IF EXISTS {phpbb2drupal_temp_user}");
    db_query("CREATE TABLE {phpbb2drupal_temp_user} (
      user_id mediumint(8) DEFAULT '0' NOT NULL,
      uid INTEGER UNSIGNED DEFAULT '0' NOT NULL,
      KEY user_id (user_id))"  
    );

    # create profile fields for icq, aim, msn...etc
    db_query("INSERT INTO {profile_fields} (title, name, explanation, category, page, type, weight, required, register, visibility, options) VALUES ('YIM','user_yim','','Contact','','textfield',0,0,1,2,''),('AIM','user_aim','','Contact','','textfield',0,0,1,2,''),('MSN','user_msnm','','Contact','','textfield',0,0,1,2,''),('icq','user_icq','','Contact','','textfield',0,0,1,2,''),('Website','user_website','','Contact','','textfield',0,0,1,2,''),('Location','user_from','','Personal','','textfield',0,0,1,2,''),('Occupation','user_occ','','Personal','','textfield',0,0,1,2,''),('Interests','user_interests','','Personal','','textfield',0,0,1,2,'')");

    variable_set('phpbb2drupal_import_user_started', 1);
  }

  # fetch all the users from PHPBB
  db_set_active('phpbb');
  $phpbb2_users = db_query("SELECT * FROM {users} WHERE user_id > 2");

  while($user = db_fetch_object($phpbb2_users)) {
    $phpbb_users[] = $user;
  }

  # Insert the users into drupal
  db_set_active('default');
  foreach($phpbb_users as $obj) {
    // Make sure the user is has not already been imported
    if(!db_result(db_query("SELECT user_id FROM {phpbb2drupal_temp_user} WHERE user_id = $obj->user_id"))) {
      $obj->user_aim = strtr($obj->user_aim, "+", ' '); # PHPBB stores spaces as +, replace with ' '
      $obj->user_yim = strtr($obj->user_yim, "+", ' ');
      $obj->user_timezone = $obj->user_timezone * 60 * 60;  # Drupal stores timezones in seconds

      $user = array('name' => $obj->username,
                    'pass' => $obj->user_password,
                    'mail' => $obj->user_email,
                    'signature' => $obj->user_sig,
                    'created' => $obj->user_regdate,
                    'status' => $obj->user_active,
                    'timezone' => $obj->user_timezone, 
                    'picture' =>$obj->user_avatar,
                    'init' => $obj->user_email,
                    'roles' => array(0 => 2),                       # Authenticated User
                    'user_website' => $obj->user_website,
                    'user_from' => $obj->user_from,
                    'user_icq' => $obj->user_icq,
                    'user_aim' => $obj->user_aim,
                    'user_yim' => $obj->user_yim,
                    'user_msnm' => $obj->user_msnm,
                    'user_occ' => $obj->user_occ,
                    'user_interests' => $obj->user_interest);

      $user = phpbb2drupal_user_save($user);
    
//      print "<pre>";
//      print_r($user);
//      print "</pre>";
      
      // populate the temporary user table
      db_query("INSERT INTO {phpbb2drupal_temp_user} (user_id, uid) VALUES ($obj->user_id , $user->uid)");
    }

    // adding the admin uid so that other functions can find the admin mapping
    db_query("INSERT INTO {phpbb2drupal_temp_user} (user_id, uid) VALUES (2 , 1)");
  }

  # set the user import successful flag in the variable table
  variable_set('phpbb2drupal_import_user_successful', '1');

  $count = db_result(db_query("SELECT COUNT(*) FROM {phpbb2drupal_temp_user}"));
  print "<h3>Successfully Imported $count Users</h3>";
}

/**
 *
 * Create Forum Containers and Forums
 *
 */
function phpbb2drupal_import_categories() {

  db_set_active('default');
  
  // check if the forum database has been successfully imported
  if(variable_get('phpbb2drupal_import_forum_successful', 0)) return;
  
  // forum mapping temporary tables
  if(!variable_get('phpbb2drupal_import_forum_started', 0)) {
    db_set_active('default');
    db_query("DROP TABLE IF EXISTS {phpbb2drupal_temp_forum}");
    db_query("CREATE TABLE {phpbb2drupal_temp_forum} (
      forum_id smallint(5) UNSIGNED DEFAULT '0' NOT NULL,
      tid integer UNSIGNED DEFAULT '0' NOT NULL,
      KEY forum_id (forum_id))"  
    );
    variable_set('phpbb2drupal_import_forum_started', 1);
  }
  
  // Retrieve the vocabulary vid named "Forum"
  $forum_vid = db_result(db_query("SELECT vid FROM {vocabulary} WHERE name='$PHPBB2DRUPAL_FORUM_NAME'"));
  
  // Create the forum vid if it doesn't exist
  if(!$forum_vid) {
    $edit = array(
      'name' => $PHPBB2DRUPAL_FORUM_NAME,
      'nodes' => array(0=>'forum'),
      'required' => 1,    // forum type are required to select a forum
      'multiple' => 0,    // can't select multiple terms
      'hierarchy' => 1    // single
    );
    taxonomy_save_vocabulary($edit);
    $forum_vid = db_result(db_query("SELECT vid FROM {vocabulary} WHERE name='$PHPBB2DRUPAL_FORUM_NAME'"));
  }

  // Get Categories/Forums from PHPBB
  db_set_active('phpbb');
  $category_results = db_query("SELECT * FROM {categories} ORDER BY cat_order");
  while($category_result = db_fetch_object($category_results)) {

    $forums_results = db_query("SELECT * FROM {forums} WHERE cat_id = $category_result->cat_id");
    
    $phpbb2_forums = array();  # reinitialize the temp var not to include it multiple times
    while($forum_result = db_fetch_object($forums_results)) {
      //$phpbb2_categories[$category_result->cat_id]['forums'][] = $forums_result;
      $phpbb2_forums[$forum_result->forum_id] = $forum_result;
    }

    $phpbb2_categories[$category_result->cat_id] = array_merge($category_result, array('forums' => $phpbb2_forums));
  }

//  print "<pre>";
//  print_r($phpbb2_categories);
//  print "</pre>";

  # Insert the Containers / Forum into Drupal
  db_set_active('default');

  # Insert the Containers
  $container_order = -10;
  foreach($phpbb2_categories as $container) {
    $edit = array('name' => $container['cat_title'],
                  'vid' => $forum_vid,
                  'description' => '',
                  'weight' => $container_order);

    $edit = taxonomy_save_term($edit);    
    //print_r($edit);

    # serialize the forum containers
    $containers = variable_get('forum_containers', array());
    $containers[] = $edit['tid'];
    variable_set('forum_containers', $containers); 
   
    # Insert the Forums
    $forum_order = -10;
    foreach($container['forums'] as $forum) {
      // Make sure the forum/term is has not already been imported
      if(!db_result(db_query("SELECT forum_id FROM {phpbb2drupal_temp_forum} WHERE forum_id = $forum->forum_id"))) {
        $forum_edit = array('name' => $forum->forum_name,
                      'vid' =>$forum_vid,
                      'description' => $forum->forum_desc,
                      'weight' => $forum_order,
                      'parent' => array(0=>$edit['tid']));

        $forum_edit = taxonomy_save_term($forum_edit);
        
        $forum_order++;
        $tid = $forum_edit['tid'];
       
        db_query("INSERT INTO {phpbb2drupal_temp_forum} (forum_id, tid) VALUES ($forum->forum_id, $tid)");
      }
    }

    $container_order++;
  }

  # set the forums import successful flag in the variable table
  variable_set('phpbb2drupal_import_forum_successful', '1');

  $count = db_result(db_query("SELECT COUNT(*) FROM {phpbb2drupal_temp_forum}"));
  print "<h3>Successfully Imported $count forums</h3>";
}

/**
 *
 * PHPBB Topics --> Drupal Forum Nodes
 *
 */
function phpbb2drupal_import_topics() {

  db_set_active('default');

  // check if the post database has been successfully imported
  if(variable_get('phpbb2drupal_import_topic_successful', 0)) return;

  if(!variable_get('phpbb2drupal_import_topic_started', 0)) {
    db_set_active('default');
    db_query("DROP TABLE IF EXISTS {phpbb2drupal_temp_topic}");
    db_query("CREATE TABLE {phpbb2drupal_temp_topic} (
      topic_id mediumint(8) UNSIGNED DEFAULT '0' NOT NULL,
      nid integer UNSIGNED DEFAULT '0' NOT NULL,
      KEY topic_id (topic_id))"  
    );
    variable_set('phpbb2drupal_import_topic_started', 1);
  }

  # Get All topics from PHPBB
  db_set_active('phpbb');
  $phpbb2_results = db_query("SELECT *
                              FROM {topics} t 
                              INNER JOIN {posts} p ON p.topic_id = t.topic_id 
                              INNER JOIN {posts_text} pt ON pt.post_id = p.post_id
                              WHERE t.topic_status <> 2 
                              GROUP BY t.topic_id");  // Moved topics are duplicates don't import

  $phpbb2_topics = array();
  while($result = db_fetch_object($phpbb2_results)) {
    $phpbb2_topics[] = $result;
  }

  # Import the topics into drupal
  db_set_active('default');
  foreach($phpbb2_topics as $topic) {
    if(!db_result(db_query("SELECT topic_id FROM {phpbb2drupal_temp_topic} WHERE topic_id = $topic->topic_id"))) {
      $uid = db_result(db_query("SELECT uid FROM {phpbb2drupal_temp_user} WHERE user_id = $topic->topic_poster"));
      $tid = db_result(db_query("SELECT tid FROM {phpbb2drupal_temp_forum} WHERE forum_id = $topic->forum_id"));

      if($topic->topic_poster == 2) {   // is the admin
        $uid = 1;
      } elseif($topic->topic_poster == -1) {
        $uid = 0;
      }

      if($topic->topic_type == 1) {
        $sticky = 1;      // sticky
        $promote = 0;
      } elseif ($topic->topic_type == 2) {
        $sticky = 0;
        $promote = 1;     // display on the front page, i.e. promote
      } else {
        $sticky = 0;
        $promote = 0;
      }

      if ($topic->topic_status == 1) { // LOCKED
        $comment = 1;  // read-only
      } else { // UNLOCKED & WATCH NOTIFIED
        $comment = 2;  // read-write
      }

      #construct the node
      $node = array(
        'type' => 'forum',
        'title' => $topic->topic_title,
        'uid' => $uid,
        'status' => 1,  // published or not - always publish
        'promote' => $promote,
        'created' => $topic->topic_time,
        'changed' => $topic->topic_time,
        'comment' => $comment,
        'moderate' => 0,
        'body' => $topic->post_text,
        'sticky' => $sticky,
        'tid' => $tid,        // which forum it belongs to
      );
    
      $node = array2object($node); // node_save requires an object form

//      print "<pre>";
//      print_r($node);
//      print "</pre>";

      $nid = node_save($node);
      taxonomy_node_save($nid, array(0 => $tid));

      if($PHPBB2DRUPAL_IMPORT_ATTACHMENTS) {
        $file_path = variable_get('file_directory_path', 'files');

        if($topic->attachment) {
          db_set_active('phpbb');
          $results = db_query("SELECT * 
                              FROM {attachments} a 
                              JOIN {attachments_desc} ad ON a. attach_id = ad.attach_id
                              WHERE attach_id = $topic->attachment");

          while($result = db_fetch_object($results)) {
            $phpbb2_files[] = $result;
          }

          db_set_active('default');
          foreach($phpbb2_files as $file) {
            $fid = db_next_id('{files}_fid');
            db_query("INSERT INTO {files} (fid, nid, filename, filepath, filemime, filesize, list) VALUES (%d, %d, '%s', '%s', '%s', %d, %d)", $fid, $nid, $file->real_filename, $file->physical_filename, $file->mimetype, $file->filesize, 1);
          }
        }
      }

      db_query("INSERT INTO {phpbb2drupal_temp_topic} (topic_id, nid) VALUES ($topic->topic_id, $nid)");
    }
  }

  # set the topic import successful flag in the variable table
  variable_set('phpbb2drupal_import_topic_successful', '1');

  $count = db_result(db_query("SELECT COUNT(*) FROM {phpbb2drupal_temp_topic}"));
  print "<h3>Successfully Imported $count topics</h3>";

}

/**
 *
 * PHPBB Posts --> Drupal Comments
 *
 */
function phpbb2drupal_import_posts() {
  db_set_active('default');

  $file_path = variable_get('file_directory_path', 'files');

  // check if the post database has been successfully imported
  if(variable_get('phpbb2drupal_import_post_successful', 0)) return;

  if(!variable_get('phpbb2drupal_import_post_started', 0)) {
    db_set_active('default');
    db_query("DROP TABLE IF EXISTS {phpbb2drupal_temp_post}");
    db_query("CREATE TABLE {phpbb2drupal_temp_post} (
      post_id mediumint(8) UNSIGNED DEFAULT '0' NOT NULL,
      cid int(10) DEFAULT '0' NOT NULL,
      KEY post_id (post_id))"  
    );
    variable_set('phpbb2drupal_import_post_started', 1);
  }

  # Get All posts from PHPBB
  db_set_active('phpbb');
  $phpbb2_results = db_query("SELECT *
                              FROM {posts} p
                              INNER JOIN {posts_text} pt ON pt.post_id = p.post_id
                              WHERE p.post_id <> p.topic_id");  

  $phpbb2_posts = array();
  while($result = db_fetch_object($phpbb2_results)) {
    $phpbb2_posts[] = $result;
  }

  # Import the posts into drupal
  db_set_active('default');
  foreach($phpbb2_posts as $post) {
    if(!db_result(db_query("SELECT post_id FROM {phpbb2drupal_temp_post} WHERE post_id = $post->post_id"))) {
      $uid = db_result(db_query("SELECT uid FROM {phpbb2drupal_temp_user} WHERE user_id = $post->poster_id"));
      $nid = db_result(db_query("SELECT nid FROM {phpbb2drupal_temp_topic} WHERE topic_id = $post->topic_id"));
      $pid = db_result(db_query("SELECT MAX(pid) FROM {comments} WHERE nid = $nid"));

      if(is_null($pid)) {
        $pid = 0;
      }

      if($post->poster_id == 2) {   // is the admin
        $uid = 1;
      } elseif($post->poster_id == -1) {
        $uid = 0;
      }

      if($uid == -1) {  // check for anonymous users
        $uid = 0;        // drupal uses 0 for anonymous users, phpbb uses -1
      } elseif($uid == 2) { // admin
        $uid = 1;
      }

      $hostname = decode_ip($post->post_ip);

      #construct the node
      $comment = array(
        'pid' => $pid,
        'nid' => $nid,
        'uid' => $uid,
        'subject' => $post->post_subject,
        'comment' => $post->post_text,
        'hostname' => $hostname,
        'timestamp' => $post->post_time
      );

//      print "<pre>";
//      print_r($comment);
//      print "</pre>";
      
      $cid = phpbb2drupal_comment_save($comment);

      if($PHPBB2DRUPAL_IMPORT_ATTACHMENTS) {
        $file_path = variable_get('file_directory_path', 'files');

        if($post->attachment) {
          db_set_active('phpbb');
          $results = db_query("SELECT * 
                              FROM {attachments} a 
                              JOIN {attachments_desc} ad ON a. attach_id = ad.attach_id
                              WHERE attach_id = $post->attachment");
          
          while($result = db_fetch_object($results)) {
            $phpbb2_files[] = $result;
          }

          db_set_active('default');
          foreach($phpbb2_files as $file) {
            $fid = db_next_id('{files}_fid');
            db_query("INSERT INTO {files} (fid, nid, filename, filepath, filemime, filesize, list) VALUES (%d, %d, '%s', '%s', '%s', %d, %d)", $fid, 0, $file->real_filename, $file->physical_filename, $file->mimetype, $file->filesize, 1);
            db_query("INSERT INTO {comment_files} (cid, fid) VALUES (%d, %d)", $cid, $fid);
          }        
        }
      }

      db_query("INSERT INTO {phpbb2drupal_temp_post} (post_id, cid) VALUES ($post->post_id, $cid)");
    }
  }

  # set the post import successful flag in the variable table
  variable_set('phpbb2drupal_import_post_successful', '1');

  $count = db_result(db_query("SELECT COUNT(*) FROM {phpbb2drupal_temp_post}"));
  print "<h3>Successfully Imported $count posts</h3>";
}

/**
 *
 * Clean UP
 *
 */
function phpbb2drupal_import_clean_up() {
  #
  # Update Drupal sequence 
  #
  db_set_active('default');
  $term_data_tid = db_result(db_query("SELECT MAX(tid) FROM {term_data}"));
  $comments_cid = db_result(db_query("SELECT MAX(cid) FROM {comments}"));
  $node_nid = db_result(db_query("SELECT MAX(nid) FROM {node} WHERE type = 'forum'"));
  $users_uid = db_result(db_query("SELECT MAX(uid) FROM {users}"));
  $files_fid = db_result(db_query("SELECT MAX(fid) FROM {files}"));

  db_query("DELETE FROM {sequences} WHERE name='term_data_tid'");
  db_query("DELETE FROM {sequences} WHERE name='comments_cid'");
  db_query("DELETE FROM {sequences} WHERE name='node_nid'");
  db_query("DELETE FROM {sequences} WHERE name='users_uid'");

  db_query("INSERT INTO {sequences} (name,id) VALUE('term_data_tid', $term_data_tid)");
  db_query("INSERT INTO {sequences} (name,id) VALUE('comments_cid', $comments_cid)");
  db_query("INSERT INTO {sequences} (name,id) VALUE('node_nid', $node_nid)");
  db_query("INSERT INTO {sequences} (name,id) VALUE('users_uid', $users_uid)");

  if($PHPBB2DRUPAL_IMPORT_ATTACHMENTS) {
    db_query("DELETE FROM {sequences} WHERE name='files_fid'");
    db_query("INSERT INTO {sequences} (name,id) VALUE('files_fid', $files_fid)");
  }

  db_query("DROP TABLE IF EXISTS {phpbb2drupal_temp_user}");
  db_query("DROP TABLE IF EXISTS {phpbb2drupal_temp_forum}");
  db_query("DROP TABLE IF EXISTS {phpbb2drupal_temp_topic}");
  db_query("DROP TABLE IF EXISTS {phpbb2drupal_temp_post}");

  variable_del('phpbb2drupal_import_user_successful');
  variable_del('phpbb2drupal_import_user_started');
  variable_del('phpbb2drupal_import_forum_successful');
  variable_del('phpbb2drupal_import_forum_started');
  variable_del('phpbb2drupal_import_topic_successful');
  variable_del('phpbb2drupal_import_topic_started');
  variable_del('phpbb2drupal_import_post_successful');
  variable_del('phpbb2drupal_import_post_started');
  variable_del('phpbb2drupal_import_attachment_successful');
  variable_del('phpbb2drupal_import_attachment_started');
  
  db_query('DELETE FROM {cache}');
}

/**
 *
 * Helper Functions
 *
 */
function phpbb2drupal_user_save($array = array(), $category = 'account') {
  // Dynamically compose a SQL query:
  $user_fields = user_fields();
  
  //$array['created'] = time();
  $array['changed'] = time();
  $array['uid'] = db_next_id('{users}_uid');

  // Note, we wait with saving the data column to prevent module-handled
  // fields from being saved there. We cannot invoke hook_user('insert') here
  // because we don't have a fully initialized user object yet.
  foreach ($array as $key => $value) {
    if ($key == 'pass') {
      $fields[] = db_escape_string($key);
      $values[] = $value;
      $s[] = "'%s'";
    }
    else if (substr($key, 0, 4) !== 'auth') {
      if (in_array($key, $user_fields)) {
        $fields[] = db_escape_string($key);
        $values[] = $value;
        $s[] = "'%s'";
      }
    }
  }
  db_query('INSERT INTO {users} ('. implode(', ', $fields) .') VALUES ('. implode(', ', $s) .')', $values);

  // Reload user roles (delete just to be safe).
  db_query('DELETE FROM {users_roles} WHERE uid = %d', $array['uid']);
  foreach ($array['roles'] as $rid) {
    db_query('INSERT INTO {users_roles} (uid, rid) VALUES (%d, %d)', $array['uid'], $rid);
  }

  // Build the initial user object.
  $user = user_load(array('uid' => $array['uid']));

  user_module_invoke('insert', $array, $user, $category);

  // Build and save the serialized data field now
  $data = array();
  foreach ($array as $key => $value) {
    if ((substr($key, 0, 4) !== 'auth') && (!in_array($key, $user_fields)) && ($value !== null)) {
      $data[$key] = $value;
    }
  }
  db_query("UPDATE {users} SET data = '%s' WHERE uid = %d", serialize($data), $user->uid);

  // Build the finished user object.
  $user = user_load(array('uid' => $array['uid']));


  // Save distributed authentication mappings
  foreach ($array as $key => $value) {
    if (substr($key, 0, 4) == 'auth') {
      $authmaps[$key] = $value;
    }
  }
  if ($authmaps) {
    user_set_authmaps($user, $authmaps);
  }

  return $user;
}

function phpbb2drupal_comment_save($edit) {

  // Here we are building the thread field.  See the comment
  // in comment_render().
  if ($edit['pid'] == 0) {
    // This is a comment with no parent comment (depth 0): we start
    // by retrieving the maximum thread level.
    $max = db_result(db_query('SELECT MAX(thread) FROM {comments} WHERE nid = %d', $edit['nid']));

    // Strip the "/" from the end of the thread.
    $max = rtrim($max, '/');

    // Next, we increase this value by one.  Note that we can't
    // use 1, 2, 3, ... 9, 10, 11 because we order by string and
    // 10 would be right after 1.  We use 1, 2, 3, ..., 9, 91,
    // 92, 93, ... instead.  Ugly but fast.
    $decimals = (string) substr($max, 0, strlen($max) - 1);
    $units = substr($max, -1, 1);
    if ($units) {
      $units++;
    }
    else {
      $units = 1;
    }

    if ($units == 10) {
      $units = '90';
    }

    // Finally, build the thread field for this new comment.
    $thread = $decimals . $units .'/';
  }
  else {
    // This is comment with a parent comment: we increase
    // the part of the thread value at the proper depth.

    // Get the parent comment:
    $parent = db_fetch_object(db_query('SELECT * FROM {comments} WHERE cid = %d', $edit['pid']));

    // Strip the "/" from the end of the parent thread.
    $parent->thread = (string) rtrim((string) $parent->thread, '/');

    // Get the max value in _this_ thread.
    $max = db_result(db_query("SELECT MAX(thread) FROM {comments} WHERE thread LIKE '%s.%%' AND nid = %d", $parent->thread, $edit['nid']));

    if ($max == '') {
      // First child of this parent.
      $thread = $parent->thread .'.1/';
    }
    else {
      // Strip the "/" at the end of the thread.
      $max = rtrim($max, '/');

      // We need to get the value at the correct depth.
      $parts = explode('.', $max);
      $parent_depth = count(explode('.', $parent->thread));
      $last = $parts[$parent_depth];

      // Next, we increase this value by one.  Note that we can't
      // use 1, 2, 3, ... 9, 10, 11 because we order by string and
      // 10 would be right after 1.  We use 1, 2, 3, ..., 9, 91,
      // 92, 93, ... instead.  Ugly but fast.
      $decimals = (string)substr($last, 0, strlen($last) - 1);
      $units = substr($last, -1, 1);
      $units++;
      if ($units == 10) {
        $units = '90';
      }

      // Finally, build the thread field for this new comment.
      $thread = $parent->thread .'.'. $decimals . $units .'/';
    } 
  }

  $edit['cid'] = db_next_id('{comments}_cid');

  $status = 0;                        // 1 - not published, 0 - published
  $format = 1;                        // 1 - filtered, 2 - PHP, 3 FULL HTML
  $score = 0;                         // 0 default value, comments get higher score depending on the author's roles
  $users = serialize(array(0 => 1));  // default value for everybody!!

  db_query("INSERT INTO {comments} (cid, nid, pid, uid, subject, comment, format, hostname, timestamp, status, score, users, thread, name, mail, homepage) VALUES (%d, %d, %d, %d, '%s', '%s', %d, '%s', %d, %d, %d, '%s', '%s', '%s', '%s', '%s')", $edit['cid'], $edit['nid'], $edit['pid'], $edit['uid'], $edit['subject'], $edit['comment'], $format, $edit['hostname'], $edit['timestamp'], $status, $score, $users, $thread, $edit['name'], $edit['mail'], $edit['homepage']);

  _comment_update_node_statistics($edit['nid']);

  return $edit['cid'];
}

// PHPBB function for decoding the user ip
function decode_ip($int_ip) {
  $hexipbang = explode('.', chunk_split($int_ip, 2, '.'));
  return hexdec($hexipbang[0]). '.' . hexdec($hexipbang[1]) . '.' . hexdec($hexipbang[2]) . '.' . hexdec($hexipbang[3]);
}

function phpbb2drupal_page_header($title) {
  $output = "<html><head><title>$title</title>";
  $output .= <<<EOF
      <link rel="stylesheet" type="text/css" media="print" href="misc/print.css" />
      <style type="text/css" title="layout" media="Screen">
        @import url("misc/drupal.css");
      </style>
EOF;
  $output .= "</head><body>";
  $output .= "<div id=\"logo\"><a href=\"http://drupal.org/\"><img src=\"misc/druplicon-small.png\" alt=\"Druplicon - Drupal logo\" title=\"Druplicon - Drupal logo\" /></a></div>";
  $output .= "<div id=\"update\"><h1>$title</h1>";
  return $output;
}

function phpbb2drupal_page_footer() {
  return "</div></body></html>";
}
?>